@@ -17,7 +17,7 @@ using namespace apolloron;
 
 const char *INKF_PROGNAME = "inkf";
 const char *INKF_BRIEF    = "Yet Another Network Kanji Filter";
-const char *INKF_VERSION  = "0.9.3";
+const char *INKF_VERSION  = "0.9.5";
 const char *INKF_COPY     = "Copyright (C) 2002-2009 Tachibanamasashi";
 const char *INKF_DEF_OUT  = "inkf.out";
 
@@ -49,9 +49,16 @@ typedef struct {
   bool flag_show_version;
   bool flag_guess;
   bool flag_overwrite;
+  bool flag_html_to_plain;
+  bool flag_hankaku_ascii;
+  bool flag_zenkaku_ascii;
+  bool flag_hankaku_katakana;
+  bool flag_zenkaku_katakana;
+  bool flag_hiragana;
+  bool flag_katakana;
   bool flag_midi;
-  char input_charset[16];
-  char output_charset[16];
+  char input_charset[32];
+  char output_charset[32];
   TMIME mime_decode;
   TMIME mime_encode;
   TLineMode line_mode;
@@ -62,6 +69,7 @@ typedef struct {
 
 TOption *set_option(TOption *option, int argc, const char **argv);
 void guess(const String &str, const char *input_charset, FILE *fpout);
+void convert(String &str, const TOption &option, FILE *fpout);
 void show_help();
 void show_version();
 void show_copy();
@@ -73,7 +81,7 @@ int main(int argc, char *argv[]) {
   FILE *fpin = stdin, *fpout = stdout;
   String tmp_str;
   char buf[4096 + 1];
-  int i, j, l;
+  int i, j, k, l;
 
   // set option
   if (set_option(&option, argc, (const char **)argv) == (TOption *)NULL) {
@@ -110,7 +118,7 @@ int main(int argc, char *argv[]) {
 
     // read from stdin
     if (option.mime_decode == MIME_NONE && option.mime_encode == MIME_NONE &&
-        !option.flag_guess && !option.flag_midi) {
+        !option.flag_guess && !option.flag_html_to_plain && !option.flag_midi) {
       while (!feof(fpin)) {
         tmp_str = "";
         while (!feof(fpin)) {
@@ -125,11 +133,7 @@ int main(int argc, char *argv[]) {
           }
         }
         if (0 < tmp_str.len()) {
-          tmp_str = tmp_str.strconv(option.input_charset, option.output_charset);
-          if (option.line_mode != LINE_MODE_NOCONVERSION) {
-            tmp_str = tmp_str.changeReturnCode(RETURN_STR[option.line_mode]);
-          }
-          fwrite(tmp_str, 1, tmp_str.len(), fpout);
+          convert(tmp_str, option, fpout);
         }
       }
     } else {
@@ -145,41 +149,9 @@ int main(int argc, char *argv[]) {
         guess(tmp_str, option.input_charset, fpout);
       } else if (option.flag_midi) {
         tmp_str = tmp_str.toMIDI();
-        fwrite(tmp_str, 1, tmp_str.binaryLength(), fpout);
+        fwrite(tmp_str.c_str(), 1, tmp_str.binaryLength(), fpout);
       } else if (0 < tmp_str.len()) {
-        if (option.mime_decode == MIME_BASE64) {
-          tmp_str = tmp_str.decodeBASE64();
-        } else if (option.mime_decode == MIME_QUOTED_PRINTABLE) {
-          tmp_str = tmp_str.decodeQuotedPrintable();
-        } else if (option.mime_decode == MIME_AUTODETECT) {
-          if (0 <= tmp_str.search("=?")) {
-            if (!strncasecmp(option.input_charset, "AUTODETECT", 10)) {
-              tmp_str = tmp_str.decodeMIME(option.input_charset, "");
-            } else {
-              tmp_str = tmp_str.decodeMIME(option.input_charset, option.input_charset);
-            }
-          } else if (tmp_str[0] != '=') {
-            tmp_str = tmp_str.decodeBASE64();
-          } else {
-            tmp_str = tmp_str.decodeQuotedPrintable();
-          }
-        }
-        tmp_str = tmp_str.strconv(option.input_charset, option.output_charset);
-        if (option.line_mode != LINE_MODE_NOCONVERSION) {
-          tmp_str = tmp_str.changeReturnCode(RETURN_STR[option.line_mode]);
-        }
-        if (option.mime_encode == MIME_BASE64) {
-          tmp_str = tmp_str.encodeBASE64();
-        } else if (option.mime_encode == MIME_QUOTED_PRINTABLE) {
-          tmp_str = tmp_str.encodeQuotedPrintable();
-        } else if (option.mime_encode == MIME_BASE64_FULL) {
-          tmp_str = tmp_str.encodeMIME(option.output_charset,
-              option.output_charset, 72, RETURN_STR[option.line_mode], 'B');
-        } else if (option.mime_encode == MIME_QUOTED_PRINTABLE_FULL) {
-          tmp_str = tmp_str.encodeMIME(option.output_charset,
-              option.output_charset, 72, RETURN_STR[option.line_mode], 'Q');
-        }
-        fwrite(tmp_str, 1, tmp_str.len(), fpout);
+        convert(tmp_str, option, fpout);
       }
     }
   } else {
@@ -197,6 +169,10 @@ int main(int argc, char *argv[]) {
         char *p;
         int host_base;
         bool ssl;
+        bool charset_autodetect;
+        bool header_end;
+        bool ischunked;
+        long chunk_size;
 #if __OPENSSL == 1
         if (!strncasecmp(option.input_filenames[i], "http://", 7)) {
 #endif
@@ -236,6 +212,10 @@ int main(int argc, char *argv[]) {
         p = strchr(option.input_filenames[i] + host_base, '/');
         strncpy(path, p?p:"/", 1023);
         path[1023] = '\0';
+        charset_autodetect = false;
+        if (!strcasecmp(option.input_charset, "AUTODETECT")) {
+          charset_autodetect = true;
+        }
         tmp_socket.setTimeout(5);
         tmp_socket.connect(host, port, ssl);
         tmp_socket.send("GET ");
@@ -244,72 +224,84 @@ int main(int argc, char *argv[]) {
         tmp_socket.send("HOST: ");
         tmp_socket.send(host);
         tmp_socket.send("\r\n");
+        tmp_socket.send("User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; ");
+        tmp_socket.send(INKF_PROGNAME);
+        tmp_socket.send(" ");
+        tmp_socket.send(INKF_VERSION);
+        tmp_socket.send(")\r\n");
         tmp_socket.send("CONNECTION: close\r\n\r\n");
         tmp_str = "";
+        header_end = false;
+        ischunked = false;
+        chunk_size = -1;
         while (tmp_socket.error() == 0 && 0 < tmp_socket.receiveLine().len()) {
-          tmp_str += tmp_socket.receivedData();
+          char *endptr;
+
+          if (!header_end &&
+              !strncasecmp(tmp_socket.receivedData().c_str(), "Transfer-Encoding", 17) &&
+              18 <= tmp_socket.receivedData().searchCase("chunked")) {
+            ischunked = true;
+          }
+
+          if (header_end && ischunked) {
+            chunk_size = strtol(tmp_socket.receivedData().c_str(), &endptr, 16);
+            if (chunk_size <= 0) {
+              continue;
+            }
+            tmp_socket.receive(chunk_size);
+          }
+
+          if (charset_autodetect && 0 <= (l = tmp_socket.receivedData().searchCase("charset="))) {
+            char charset[32], c;
+            k = 0;
+            while (k < 31) {
+              c = tmp_socket.receivedData()[l+8+k];
+              if (!(isalnum(c) || c == '-' || c == '_')) {
+                break;
+              }
+              charset[k] = c;
+              k++;
+            }
+            charset[k] = '\0';
+            if (0 < k) {
+              strcpy(option.input_charset, charset);
+              charset_autodetect = false;
+            }
+          }
+
+          if (header_end) {
+            tmp_str += tmp_socket.receivedData();
+          }
+
+          if (!header_end && (tmp_socket.receivedData()[0] == '\n' || tmp_socket.receivedData() == "\r\n")) {
+            header_end = true;
+          }
         }
         tmp_socket.clear();
-        l = tmp_str.search("\r\n\r\n");
-        if (0 <= l) {
-          tmp_str = tmp_str.mid(l + 4);
-        }
       } else {
         tmp_str.loadFile(option.input_filenames[i]);
       }
       if ((option.output_filename == NULL || option.output_filename[0] == '\0') &&
           option.flag_overwrite) {
-          fpout = fopen(option.input_filenames[i], "wb");
-          if (fpout == (FILE *)NULL) {
-            fprintf(stderr, "Cannot open the output file '%s'.\n",
-                option.input_filenames[i]);
-            continue;
-          }
+        fpout = fopen(option.input_filenames[i], "wb");
+        if (fpout == (FILE *)NULL) {
+          fprintf(stderr, "Cannot open the output file '%s'.\n",
+              option.input_filenames[i]);
+          continue;
+        }
       }
       if (option.flag_guess) {
         guess(tmp_str, option.input_charset,  fpout);
         break;
       } else if (option.flag_midi) {
         tmp_str = tmp_str.toMIDI();
-        fwrite(tmp_str, 1, tmp_str.binaryLength(), fpout);
+        fwrite(tmp_str.c_str(), 1, tmp_str.binaryLength(), fpout);
         if (!((option.output_filename == NULL || option.output_filename[0] == '\0') &&
             option.flag_overwrite)) {
           break;
         }
       } else if (0 < tmp_str.len()) {
-        if (option.mime_decode == MIME_BASE64) {
-          tmp_str = tmp_str.decodeBASE64();
-        } else if (option.mime_decode == MIME_QUOTED_PRINTABLE) {
-          tmp_str = tmp_str.decodeQuotedPrintable();
-        } else if (option.mime_decode == MIME_AUTODETECT) {
-          if (0 <= tmp_str.search("=?")) {
-            if (!strncasecmp(option.input_charset, "AUTODETECT", 10)) {
-              tmp_str = tmp_str.decodeMIME(option.input_charset, "");
-            } else {
-              tmp_str = tmp_str.decodeMIME(option.input_charset, option.input_charset);
-            }
-          } else if (tmp_str[0] != '=') {
-            tmp_str = tmp_str.decodeBASE64();
-          } else {
-            tmp_str = tmp_str.decodeQuotedPrintable();
-          }
-        }
-        tmp_str = tmp_str.strconv(option.input_charset, option.output_charset);
-        if (option.line_mode != LINE_MODE_NOCONVERSION) {
-          tmp_str = tmp_str.changeReturnCode(RETURN_STR[option.line_mode]);
-        }
-        if (option.mime_encode == MIME_BASE64) {
-          tmp_str = tmp_str.encodeBASE64();
-        } else if (option.mime_encode == MIME_QUOTED_PRINTABLE) {
-          tmp_str = tmp_str.encodeQuotedPrintable();
-        } else if (option.mime_encode == MIME_BASE64_FULL) {
-          tmp_str = tmp_str.encodeMIME(option.output_charset,
-              option.output_charset, 72, RETURN_STR[option.line_mode], 'B');
-        } else if (option.mime_encode == MIME_QUOTED_PRINTABLE_FULL) {
-          tmp_str = tmp_str.encodeMIME(option.output_charset,
-              option.output_charset, 72, RETURN_STR[option.line_mode], 'Q');
-        }
-        fwrite(tmp_str, 1, tmp_str.len(), fpout);
+        convert(tmp_str, option, fpout);
       }
       if ((option.output_filename == NULL || option.output_filename[0] == '\0') &&
           option.flag_overwrite) {
@@ -346,6 +338,13 @@ TOption *set_option(TOption *option, int argc, const char **argv) {
   option->flag_show_version = false;
   option->flag_guess = false;
   option->flag_overwrite = false;
+  option->flag_html_to_plain = false;
+  option->flag_hankaku_ascii = false;
+  option->flag_zenkaku_ascii = false;
+  option->flag_hankaku_katakana = false;
+  option->flag_zenkaku_katakana = false;
+  option->flag_hiragana = false;
+  option->flag_katakana = false;
   option->flag_midi = false;
   strcpy(option->input_charset, "AUTODETECT");
   option->output_charset[0] = '\0';
@@ -375,6 +374,20 @@ TOption *set_option(TOption *option, int argc, const char **argv) {
         option->output_charset[15] = '\0';
       } else if (!strcasecmp(argv[i], "--overwrite")) {
         option->flag_overwrite = true;
+      } else if (!strcasecmp(argv[i], "--html-to-plain")) {
+        option->flag_html_to_plain = true;
+      } else if (!strcasecmp(argv[i], "--hankaku-ascii")) {
+        option->flag_hankaku_ascii = true;
+      } else if (!strcasecmp(argv[i], "--zenkaku-ascii")) {
+        option->flag_zenkaku_ascii = true;
+      } else if (!strcasecmp(argv[i], "--hankaku-katakana")) {
+        option->flag_hankaku_katakana = true;
+      } else if (!strcasecmp(argv[i], "--zenkaku-katakana")) {
+        option->flag_zenkaku_katakana = true;
+      } else if (!strcasecmp(argv[i], "--hiragana")) {
+        option->flag_hiragana = true;
+      } else if (!strcasecmp(argv[i], "--katakana")) {
+        option->flag_katakana = true;
       } else if (!strcasecmp(argv[i], "--midi")) {
         option->flag_midi = true;
       } else {
@@ -462,6 +475,9 @@ TOption *set_option(TOption *option, int argc, const char **argv) {
               j++;
             }
             break;
+          case 'Z':
+            option->flag_hankaku_ascii = true;
+            break;
           case 'O':
             o_param_found = true;
             break;
@@ -554,6 +570,62 @@ void guess(const String &str, const char *input_charset, FILE *fpout) {
 }
 
 
+void convert(String &str, const TOption &option, FILE *fpout) {
+  if (option.mime_decode == MIME_BASE64) {
+    str = str.decodeBASE64();
+  } else if (option.mime_decode == MIME_QUOTED_PRINTABLE) {
+    str = str.decodeQuotedPrintable();
+  } else if (option.mime_decode == MIME_AUTODETECT) {
+    if (0 <= str.search("=?")) {
+      if (!strncasecmp(option.input_charset, "AUTODETECT", 10)) {
+        str = str.decodeMIME(option.input_charset, "");
+      } else {
+        str = str.decodeMIME(option.input_charset, option.input_charset);
+      }
+    } else if (str[0] != '=') {
+      str = str.decodeBASE64();
+    } else {
+      str = str.decodeQuotedPrintable();
+    }
+  }
+  str = str.strconv(option.input_charset, option.output_charset);
+  if (option.flag_html_to_plain) {
+    str = str.convertHTMLToPlain(option.output_charset, option.output_charset);
+  }
+  if (option.flag_hankaku_ascii) {
+    str = str.changeWidth("a", option.output_charset, option.output_charset);
+  } else if (option.flag_zenkaku_ascii) {
+    str = str.changeWidth("A", option.output_charset, option.output_charset);
+  }
+  if (option.flag_hiragana) {
+    str = str.changeWidth("HI", option.output_charset, option.output_charset);
+  } else if (option.flag_katakana) {
+    str = str.changeWidth("J", option.output_charset, option.output_charset);
+  }
+  if (option.flag_hankaku_katakana) {
+    str = str.changeWidth("k", option.output_charset, option.output_charset);
+  } else if (option.flag_zenkaku_katakana) {
+    str = str.changeWidth("K", option.output_charset, option.output_charset);
+  }
+  if (option.line_mode != LINE_MODE_NOCONVERSION) {
+    str = str.changeReturnCode(RETURN_STR[option.line_mode]);
+  }
+  if (option.mime_encode == MIME_BASE64) {
+    str = str.encodeBASE64();
+  } else if (option.mime_encode == MIME_QUOTED_PRINTABLE) {
+    str = str.encodeQuotedPrintable();
+  } else if (option.mime_encode == MIME_BASE64_FULL) {
+    str = str.encodeMIME(option.output_charset,
+        option.output_charset, 72, RETURN_STR[option.line_mode], 'B');
+  } else if (option.mime_encode == MIME_QUOTED_PRINTABLE_FULL) {
+    str = str.encodeMIME(option.output_charset,
+        option.output_charset, 72, RETURN_STR[option.line_mode], 'Q');
+  }
+
+  fwrite(str.c_str(), 1, str.len(), fpout);
+}
+
+
 void show_help() {
   fprintf(stderr, 
       "Usage:  %s -[flags] [--] [in file/in URL] .. [out file for -O flag]\n"
@@ -563,25 +635,33 @@ void show_help() {
       "                EUC-KR, GB2312, BIG5\n"
       " m[BQN0]        MIME decode [B:base64,Q:quoted,N:non-strict,0:no decode]\n"
       " M[BQ]          MIME encode [B:base64 Q:quoted]\n"
+      " Z              Convert JISX0208 Alphabet to ASCII\n"
       " O              Output to File (DEFAULT '%s')\n"
       " L[uwm]         Line mode u:LF w:CRLF m:CR (DEFAULT noconversion)\n"
       " --ic=<encoding>        Specify the input encoding\n"
       " --oc=<encoding>        Specify the output encoding\n"
       "  *supported encoding is as follows:\n"
-      "   ISO-2022-JP (ISO-2022-JP-*)\n"
-      "   SHIFT_JIS (CP932)\n"
+      "   ISO-2022-JP\n"
+      "   Shift_JIS (CP932)\n"
       "   EUC-JP\n"
       "   ISO-8859-[1,2,3,4,5,6,7,8,9,10,13,14,15,16]\n"
       "   KOI8-[R,U]\n"
       "   CP1251\n"
-      "   BIG5 (CP950,EUC-TW)\n"
-      "   GB2312 (GBK,GB18030,EUC-CN)\n"
+      "   CP1252\n"
+      "   BIG5 (CP950)\n"
+      "   GB2312 (GBK,GB18030)\n"
       "   EUC-KR (CP949)\n"
-      "   UTF-8 (UTF8)\n"
-      "   UTF-7 (UTF7)\n"
+      "   UTF-8\n"
+      "   UTF-7\n"
       "   UTF-7-IMAP\n"
       "   AUTODETECT\n"
       "   AUTODETECT_JP\n"
+      " --hiragana --katakana  Hiragana/Katakana Conversion\n"
+      " --zenkaku-ascii        Convert ASCII to JISX0208 Alphabet\n"
+      " --hankaku-ascii        Convert JISX0208 Alphabet to ASCII\n"
+      " --zenkaku-katakana     Convert Hankaku Kana to JISX0208 Katakana\n"
+      " --hankaku-katakana     Convert JISX0208 Katakana to Hankaku Kana\n"
+      " --html-to-plain        Convert HTML to Plain Text\n"
       " -g --guess     Guess the input code\n"
       " --midi         Create MIDI object from MML like music sequencial text\n"
       " --overwrite    Overwrite original listed files by filtered result\n"
