@@ -2,7 +2,7 @@
 /*! @file apolloron.h
     @brief Header file of libapolloron.
     @author Tachibanamasashi, Apolloron Project.
-    @date 2009-02-01
+    @date 2009-02-15
  ******************************************************************************/
 
 #ifndef _APOLLORON_H_
@@ -16,7 +16,7 @@ class Keys;
 
 /*! @brief Version of libapolloron.
  */
-#define LIBAPOLLORON_VERSION "0.42.2"
+#define LIBAPOLLORON_VERSION "0.42.4"
 
 
 /*----------------------------------------------------------------------------*/
@@ -211,6 +211,7 @@ public:
 
   // Search
   virtual long search(const String &key, long pos = 0L) const;
+  virtual long searchCase(const String &key, long pos = 0L) const;
   virtual long searchChar(const char c, long pos = 0L) const;
 
   // Format
@@ -250,6 +251,9 @@ public:
   // Plain text or HTML template convertion
   virtual String& evalText(Keys replace_keys) const;
 
+  // HTML to plain text
+  virtual String& convertHTMLToPlain(const char *src_charset = "UTF-8", const char *dest_charset = "UTF-8") const;
+
   // Character code conversion
   virtual String& strconv(const char *src_charset, const char *dest_charset) const;
 
