@@ -24,7 +24,7 @@
 #include "charset/table_unicode_big5.h"
 #include "charset/table_big5_unicode.h"
 #include "charset/table_iso8859_unicode.h"
-#include "charset/table_koi8_unicode.h"
+#include "charset/table_europe_unicode.h"
 #include "charset/table_charwidth.h"
 #include "charset.h"
 
@@ -48,13 +48,14 @@ const char *STR_ISO8859_16 = "ISO-8859-16";
 const char *STR_KOI8_R     = "KOI8-R";
 const char *STR_KOI8_U     = "KOI8-U";
 const char *STR_CP1251     = "CP1251";
+const char *STR_CP1252     = "CP1252";
 const char *STR_UTF8       = "UTF-8";
 const char *STR_UTF7       = "UTF-7";
 const char *STR_UTF7_IMAP  = "UTF-7-IMAP";
 const char *STR_SJIS       = "CP932";
 const char *STR_JIS        = "ISO-2022-JP";
 const char *STR_EUCJP      = "EUC-JP";
-const char *STR_ASCII      = "ASCII";
+const char *STR_ASCII      = "US-ASCII";
 const char *STR_AUTOJP     = "AUTODETECT_JP";
 const char *STR_EUCKR      = "CP949";
 const char *STR_BIG5       = "CP950";
@@ -88,8 +89,10 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
   buf = NULL;
   converted = 0;
 
-  if (src_charset == NULL) {
+  if (src_charset == NULL || src_charset[0] == '\0') {
     src_ch = STR_AUTO;
+  } else if (!strncasecmp(src_charset, "ASCII", 5) || !strncasecmp(src_charset, "US-ASCII", 8)) {
+    src_ch = STR_ASCII;
   } else if (!strncasecmp(src_charset, "EUC-JP", 6) || !strncasecmp(src_charset, "EUCJP", 5)) {
     src_ch = STR_EUCJP;
   } else if (!strncasecmp(src_charset, "SHIFT_JIS", 9) || !strncasecmp(src_charset, "SHIFT-JIS", 9) ||
@@ -118,7 +121,10 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
       !strcasecmp(src_charset, "CP949") || !strcasecmp(src_charset, "WINDOWS-949") ||
       !strcasecmp(src_charset, "MS949")) {
     src_ch = STR_EUCKR;
-  } else if (!strncasecmp(src_charset, "GB", 2) || !strncasecmp(src_charset, "EUC-CN", 6) || !strncasecmp(src_charset, "EUCCN", 5)) {
+  } else if (!strncasecmp(src_charset, "GB", 2) ||
+      !strncasecmp(src_charset, "EUC-CN", 6) || !strncasecmp(src_charset, "EUCCN", 5) ||
+      !strcasecmp(src_charset, "CP936") || !strcasecmp(src_charset, "WINDOWS-936") ||
+      !strcasecmp(src_charset, "MS936")) {
     src_ch = STR_GBK;
   } else if (!strcasecmp(src_charset, "BIG5") || !strncasecmp(src_charset, "EUC-TW", 6) ||
     !strncasecmp(src_charset, "EUCTW", 5) || !strcasecmp(src_charset, "CP950") ||
@@ -127,9 +133,16 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
   } else if (!strcasecmp(src_charset, "CP1251") || !strcasecmp(src_charset, "WINDOWS-1251") ||
       !strcasecmp(src_charset, "MS1251")) {
     src_ch = STR_CP1251;
-  } else if (!strncasecmp(src_charset, "KOI8-U", 6)) {
+  } else if (!strcasecmp(src_charset, "CP1252") || !strcasecmp(src_charset, "WINDOWS-1252") ||
+      !strcasecmp(src_charset, "MS1252")) {
+    src_ch = STR_CP1252;
+  } else if (!strncasecmp(src_charset, "KOI8-U", 6) ||
+      !strcasecmp(src_charset, "CP21866") || !strcasecmp(src_charset, "WINDOWS-21866") ||
+      !strcasecmp(src_charset, "MS21866")) {
     src_ch = STR_KOI8_U;
-  } else if (!strncasecmp(src_charset, "KOI8", 4)) {
+  } else if (!strncasecmp(src_charset, "KOI8", 4) ||
+      !strcasecmp(src_charset, "CP20866") || !strcasecmp(src_charset, "WINDOWS-20866") ||
+      !strcasecmp(src_charset, "MS20866")) {
     src_ch = STR_KOI8_R;
   } else {
     src_ch = src_charset;
@@ -137,6 +150,8 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
 
   if (dest_charset == NULL) {
     dest_ch = "";
+  } else if (!strncasecmp(dest_charset, "ASCII", 5) || !strncasecmp(dest_charset, "US-ASCII", 8)) {
+    dest_ch = STR_ASCII;
   } else if (!strncasecmp(dest_charset, "EUC-JP", 6) || !strncasecmp(dest_charset, "EUCJP", 5)) {
     dest_ch = STR_EUCJP;
   } else if (!strncasecmp(dest_charset, "SHIFT_JIS", 9) || !strncasecmp(dest_charset, "SHIFT-JIS", 9) ||
@@ -155,7 +170,10 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
       !strcasecmp(dest_charset, "CP949") || !strcasecmp(dest_charset, "WINDOWS-949") ||
       !strcasecmp(dest_charset, "MS949")) {
     dest_ch = STR_EUCKR;
-  } else if (!strncasecmp(dest_charset, "GB", 2) || !strncasecmp(dest_charset, "EUC-CN", 6) || !strncasecmp(src_charset, "EUCCN", 5)) {
+  } else if (!strncasecmp(dest_charset, "GB", 2) ||
+      !strncasecmp(dest_charset, "EUC-CN", 6) || !strncasecmp(src_charset, "EUCCN", 5) ||
+      !strcasecmp(dest_charset, "CP936") || !strcasecmp(dest_charset, "WINDOWS-936") ||
+      !strcasecmp(dest_charset, "MS936")) {
     dest_ch = STR_GBK;
   } else if (!strcasecmp(dest_charset, "BIG5") || !strncasecmp(dest_charset, "EUC-TW", 6) ||
     !strncasecmp(dest_charset, "EUCTW", 5) || !strcasecmp(dest_charset, "CP950") ||
@@ -164,9 +182,16 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
   } else if (!strcasecmp(dest_charset, "CP1251") || !strcasecmp(dest_charset, "WINDOWS-1251") ||
       !strcasecmp(dest_charset, "MS1251")) {
     dest_ch = STR_CP1251;
-  } else if (!strncasecmp(dest_charset, "KOI8-U", 6)) {
+  } else if (!strcasecmp(dest_charset, "CP1252") || !strcasecmp(dest_charset, "WINDOWS-1252") ||
+      !strcasecmp(dest_charset, "MS1252")) {
+    dest_ch = STR_CP1252;
+  } else if (!strncasecmp(dest_charset, "KOI8-U", 6) ||
+      !strcasecmp(dest_charset, "CP21866") || !strcasecmp(dest_charset, "WINDOWS-21866") ||
+      !strcasecmp(dest_charset, "MS21866")) {
     dest_ch = STR_KOI8_U;
-  } else if (!strncasecmp(dest_charset, "KOI8", 4)) {
+  } else if (!strncasecmp(dest_charset, "KOI8", 4) ||
+      !strcasecmp(dest_charset, "CP20866") || !strcasecmp(dest_charset, "WINDOWS-20866") ||
+      !strcasecmp(dest_charset, "MS20866")) {
     dest_ch = STR_KOI8_R;
   } else {
     dest_ch = "";
@@ -314,13 +339,16 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
       buf = utf8_to_iso8859(str, num);
       converted = 1;
     } else if (!strcasecmp(dest_ch, "KOI8-U")) {
-      buf = utf8_to_koi8(str, 'U');
+      buf = utf8_to_europe(str, 'U');
       converted = 1;
     } else if (!strcasecmp(dest_ch, "KOI8-R")) {
-      buf = utf8_to_koi8(str, 'R');
+      buf = utf8_to_europe(str, 'R');
       converted = 1;
     } else if (!strcasecmp(dest_ch, "CP1251")) {
-      buf = utf8_to_koi8(str, 'W');
+      buf = utf8_to_europe(str, '1');
+      converted = 1;
+    } else if (!strcasecmp(dest_ch, "CP1252")) {
+      buf = utf8_to_europe(str, '2');
       converted = 1;
     }
   } else if (!strcasecmp(src_ch, STR_AUTOJP) || !strcasecmp(src_ch, STR_AUTO)) {
@@ -379,17 +407,22 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
     }
   } else if (!strcasecmp(src_ch, "KOI8-U")) {
     if (!strcasecmp(dest_ch, STR_UTF8)) {
-      buf = koi8_to_utf8(str, 'U');
+      buf = europe_to_utf8(str, 'U');
       converted = 1;
     }
   } else if (!strcasecmp(src_ch, "KOI8-R")) {
     if (!strcasecmp(dest_ch, STR_UTF8)) {
-      buf = koi8_to_utf8(str, 'R');
+      buf = europe_to_utf8(str, 'R');
       converted = 1;
     }
   } else if (!strcasecmp(src_ch, "CP1251")) {
     if (!strcasecmp(dest_ch, STR_UTF8)) {
-      buf = koi8_to_utf8(str, 'W');
+      buf = europe_to_utf8(str, '1');
+      converted = 1;
+    }
+  } else if (!strcasecmp(src_ch, "CP1252")) {
+    if (!strcasecmp(dest_ch, STR_UTF8)) {
+      buf = europe_to_utf8(str, '2');
       converted = 1;
     }
   }
@@ -862,23 +895,25 @@ char* utf8_to_iso8859(const char* str, int iso8859_num) {
 }
 
 
-/*! Convert character set from KOI8-x to UTF-8
+/*! Convert character set from KOI8-x/CP1251/CP1252 to UTF-8
     @param str          KOI-8-x text
-    @param ch           x of KOI8-x. 'w' for CP1251
+    @param ch           x of KOI8-x. '1' for CP1251, '2' for CP1252
     @return Converted text  (UTF-8 text)
  */
-char* koi8_to_utf8(const char* str, char ch) {
+char* europe_to_utf8(const char* str, char ch) {
   char *buf;
   long i, j, length, length2;
   unsigned char a0, a1;
-  const unsigned char (*koi8_map)[KOI8_UNICODE_TABLE_MAX][2];
+  const unsigned char (*eur_map)[EUROPE_UNICODE_TABLE_MAX][2];
 
-  if (ch == 'W' || ch == 'w') {
-    koi8_map = &cp1251_unicode;
+  if (ch == '1') {
+    eur_map = &cp1251_unicode;
+  } else if (ch == '2') {
+    eur_map = &cp1252_unicode;
   } else if (ch == 'U' || ch == 'u') {
-    koi8_map = &koi8_u_unicode;
+    eur_map = &koi8_u_unicode;
   } else {
-    koi8_map = &koi8_r_unicode;
+    eur_map = &koi8_r_unicode;
   }
 
   buf = NULL;
@@ -889,13 +924,13 @@ char* koi8_to_utf8(const char* str, char ch) {
     length2 = length*3;
     buf = new char[length2 + 1];
 
-    // KOI8-x to UTF-8
+    // KOI8-x/CP1251/CP1252 to UTF-8
     i = 0;
     j = 0;
     while (i < length) {
       if ((unsigned char)0x80 <= (unsigned char)str[i]) {
-        a0 = (*koi8_map)[(unsigned char)str[i] - (unsigned char)0x80][0];
-        a1 = (*koi8_map)[(unsigned char)str[i] - (unsigned char)0x80][1];
+        a0 = (*eur_map)[(unsigned char)str[i] - (unsigned char)0x80][0];
+        a1 = (*eur_map)[(unsigned char)str[i] - (unsigned char)0x80][1];
 	if (a0 == (unsigned char)0x00) {
           buf[j++] = a1;
 	} else if (a0 <= (unsigned char)0x07) {
@@ -918,23 +953,25 @@ char* koi8_to_utf8(const char* str, char ch) {
 }
 
 
-/*! Convert character set from UTF-8 to KOI8-x
+/*! Convert character set from UTF-8 to KOI8-x/CP1251/CP1252
     @param str          UTF-8 text
-    @param ch           x of KOI8-x. 'w' for CP1251
-    @return Converted text (KOI8-x text)
+    @param ch           x of KOI8-x. '1' for CP1251, '2' for CP1252
+    @return Converted text (KOI8-x/CP1251/CP1252 text)
  */
-char* utf8_to_koi8(const char* str, char ch) {
+char* utf8_to_europe(const char* str, char ch) {
   char *buf;
   long i, j, length, length2;
   unsigned char a0, a1, a2;
-  const unsigned char (*koi8_map)[KOI8_UNICODE_TABLE_MAX][2];
+  const unsigned char (*eur_map)[EUROPE_UNICODE_TABLE_MAX][2];
 
-  if (ch == 'W' || ch == 'w') {
-    koi8_map = &cp1251_unicode;
+  if (ch == '1') {
+    eur_map = &cp1251_unicode;
+  } else if (ch == '2') {
+    eur_map = &cp1252_unicode;
   } else if (ch == 'U' || ch == 'u') {
-    koi8_map = &koi8_u_unicode;
+    eur_map = &koi8_u_unicode;
   } else {
-    koi8_map = &koi8_r_unicode;
+    eur_map = &koi8_r_unicode;
   }
 
   buf = NULL;
@@ -945,7 +982,7 @@ char* utf8_to_koi8(const char* str, char ch) {
     length2 = length*3;
     buf = new char[length2 + 1];
 
-    // UTF-8 to KOI8-x
+    // UTF-8 to KOI8-x/CP1251/CP1252
     i = 0;
     j = 0;
     while (i < length && j <= length2) {
@@ -984,15 +1021,15 @@ char* utf8_to_koi8(const char* str, char ch) {
         }
 
         n = 0;
-        while (n < KOI8_UNICODE_TABLE_MAX) {
-          if ((*koi8_map)[n][0] == a0 && (*koi8_map)[n][1] == a1) {
+        while (n < EUROPE_UNICODE_TABLE_MAX) {
+          if ((*eur_map)[n][0] == a0 && (*eur_map)[n][1] == a1) {
             buf[j++] = (unsigned char)(0x80 + n);
             i += (utf8_len(str[i])-1);
             break;
           }
           n++;
         }
-        if (KOI8_UNICODE_TABLE_MAX <= n) {
+        if (EUROPE_UNICODE_TABLE_MAX <= n) {
           buf[j++] = '?';
         }
       } else {
